searchData={"items":[{"type":"module","title":"NoNoncense","doc":"Generate locally unique nonces (number-only-used-once) in distributed Elixir.\n\nLocally unique means that the nonces are unique within your application/database/domain, as opposed to globally unique nonces that are unique across applications/databases/domains, like UUIDs.","ref":"NoNoncense.html"},{"type":"module","title":"Nonce types - NoNoncense","doc":"Several types of nonces can be generated, although they share their basic composition. The first 42 bits are a millisecond-precision timestamp (allows for ~139 years of operation), relative to the NoNoncense epoch (2025-01-01 00:00:00 UTC) by default. The next 9 bits are the machine ID (allows for 512 machines). The remaining bits are a per-machine counter.","ref":"NoNoncense.html#module-nonce-types"},{"type":"module","title":"Counter nonces - NoNoncense","doc":"- Features: unique.\n- Generation rate: very high.\n- Info leak: medium (machine init time, creation order).\n- Crypto: technically suitable for block ciphers in modes that require an IV that is unique but not necessarily unpredictable (like CTR, OFB, CCM, and GCM), and some streaming ciphers. Only when some info leak is acceptable.\n\n96/128 bits counter nonces can be generated at a practically unlimited rate of >= 2^45 nonces per ms per machien. With 64-bits nonces, an overflow of the 13 counter bits will trigger a timestamp increase by 1ms (the timestamp effectively functions as an extended counter after initialization). The maximum *sustained* rate is 8M/s per machine. Because the timestamp can't exceed the actual time (that would break the uniqueness guarantee), new nonce generation throttles if the nonce timestamp/counter catches up to the actual time. In practice, that will probably never happen, and nonces will be generated at a higher rate. For example, if the first nonce is generated 10 seconds after initialization, 10K milliseconds have been \"saved up\" to generate 80M 64-bit nonces as quickly as hardware will allow. Benchmarking shows rates in the tens of millions per second are attainable.","ref":"NoNoncense.html#module-counter-nonces"},{"type":"module","title":"Sortable nonces (Snowflake IDs) - NoNoncense","doc":"- Features: unique, time-sortable.\n- Generation rate: high.\n- Info leak: high (creation time, creation order).\n- Crypto: not recommended. They leak more info than counter nonces but are slower to generate.\n\nSortable nonces have an accurate creation timestamp, instead of counter nonces' hybrid init time + counter construction. This makes them equivalent to [Snowflake IDs](https://en.wikipedia.org/wiki/Snowflake_ID), apart from the slightly altered bit distribution of NoNoncense nonces (42 instead of 41 timestamp bits, 9 instead of 10 ID bits, no unused bit).\n\nThis has some implications. Again, 96/128-bits sortable nonces can be generated as quickly as your hardware can go. However, the 64-bits variant can be generated at 8M/s per machine and can't ever burst beyond that (the \"saving up seconds\" mechanic of counter nonces does not apply here). This should of course be plenty for most applications.","ref":"NoNoncense.html#module-sortable-nonces-snowflake-ids"},{"type":"module","title":"Encrypted nonces - NoNoncense","doc":"- Features: unique, unpredictable.\n- Generation rate: medium (scales well with CPU cores).\n- Info leak: none.\n- Crypto: same as counter nonces, but no info leaks. Additionally, suitable for block cipher modes that require unpredictable IVs, like CBC and CFB.\n\nThese nonces are encrypted in a way that preserves their uniqueness, but they are unpredictable and don't leak information. An important caveat is that 96-bits encrypted nonces leak message ordering info in their last 32 bits. For more info, see [encrypted nonces](#module-encrypted-nonces).","ref":"NoNoncense.html#module-encrypted-nonces"},{"type":"module","title":"Usage - NoNoncense","doc":"Note that `NoNoncense` is not a GenServer. Instead it stores its initial state using `m::persistent_term` and its internal counter using `m::atomics`. Because `m::persistent_term` triggers a garbage collection cycle on writes, it is recommended to initialize your `NoNoncense` instance(s) at application start, when there is hardly any garbage to collect.\n\n    # lib/my_app/application.ex\n    # generate a machine ID, start conflict guard and initialize a NoNoncense instance\n    defmodule MyApp.Application do\n      use Application\n\n      def start(_type, _args) do\n        machine_id = NoNoncense.MachineId.id!(node_list: [:\"myapp@127.0.0.1\"])\n        :ok = NoNoncense.init(machine_id: machine_id)\n\n        children =\n          [\n            # optional but recommended\n            {NoNoncense.MachineId.ConflictGuard, [machine_id: machine_id]}\n          ]\n\n        opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n        Supervisor.start_link(children, opts)\n      end\n    end\n\nThen you can generate nonces.\n\n    # generate counter nonces\n    iex> <<_::64>> = NoNoncense.nonce(64)\n    iex> <<_::96>> = NoNoncense.nonce(96)\n    iex> <<_::128>> = NoNoncense.nonce(128)\n\n    # generate sortable nonces\n    iex> <<_::64>> = NoNoncense.sortable_nonce(64)\n    iex> <<_::96>> = NoNoncense.sortable_nonce(96)\n    iex> <<_::128>> = NoNoncense.sortable_nonce(128)\n\n    # generate encrypted nonces\n    # be sure to read the NoNoncense docs before using 64/96 bits encrypted nonces\n    iex> <<_::64>> = NoNoncense.encrypted_nonce(64, :crypto.strong_rand_bytes(24))\n    iex> <<_::96>> = NoNoncense.encrypted_nonce(96, :crypto.strong_rand_bytes(24))\n    iex> <<_::128>> = NoNoncense.encrypted_nonce(128, :crypto.strong_rand_bytes(32))","ref":"NoNoncense.html#module-usage"},{"type":"module","title":"Uniqueness guarantees - NoNoncense","doc":"Nonces are guaranteed to be unique if:\n- Machine IDs are unique for each node (`NoNoncense.MachineId` and `NoNoncense.MachineId.ConflictGuard` can help there).\n- Individual machines maintain a somewhat accurate clock (specifically, the UTC clock has to have progressed between node restarts).\n- (Sortable nonces only) the machine clock has to be accurate.","ref":"NoNoncense.html#module-uniqueness-guarantees"},{"type":"module","title":"Encrypted nonces - NoNoncense","doc":"By encrypting a nonce, the timestamp, machine ID and message ordering information leak can be prevented. However, we wish to encrypt in a way that **maintains the uniqueness guarantee** of the input counter nonce. So 2^64 unique inputs should generate 2^64 unique outputs, same for the other sizes.\n\nIETF has some [wisdom to share](https://datatracker.ietf.org/doc/html/rfc8439#section-4) on the topic of nonce encryption (in the context of ChaCha20 / Poly1305 nonces):\n\n> Counters and LFSRs are both acceptable ways of generating unique nonces, as is encrypting a counter using a block cipher with a 64-bit block size such as DES. Note that it is not acceptable to use a truncation of a counter encrypted with block ciphers with 128-bit or 256-bit blocks, because such a truncation may repeat after a short time.\n\nThere are some interesting things to unpick there. Why can't we use higher ciphers with a larger block size? As it turns out, block ciphers only generate unique outputs for inputs of at least their block size (128 bits for most modern ciphers, notably AES). For example, encrypting a 64-bit nonce with AES would produce a unique 128-bit ciphertext, but that ciphertext can't be reduced back to 64 bits without losing the uniqueness property. Sadly, this also holds for the streaming modes of these ciphers, which still use blocks internally to generate the keystream. That means we can just use AES256 ECB (we only encrypt one block) for 128-bit nonces.\n\n> #### 128-bit encrypted nonces {: .tip}\n>\n> We have AES256-encrypted 128-bit nonces that are unique and indistinguishable from random noise.\n\nHowever, for 64-bit nonces we are limited to block ciphers with 64-bit block sizes. There are only a few of those in OTP's `m::crypto` module, namely DES, 3DES, and BlowFish. DES is broken and can merely be considered obfuscation at this point, despite the IETF quote (from 2018). BlowFish performs atrociously in the OTP implementation (~30 times worse than AES, dropping from ~1.8M ops/s to 60K ops/s) to the point where it can realistically form a bottleneck. 3DES seems like the least worst option, and practical attacks on it are mainly aimed at block collisions (within a message) which doesn't apply here. Still, it's old.\n\nFor 96-bit nonces there are no block ciphers whatsoever to choose from. All we can do is generate a 64-bits nonce and postfix 4 zero-bytes. That way the nonce is unique, and while the last 4 bytes are predictable, the nonce as a whole is not and no message ordering information leaks.\n\n> #### 64/96-bit encrypted nonces {: .info}\n>\n> We have 3DES-encrypted 64/96-bit nonces, which is probably good enough.","ref":"NoNoncense.html#module-encrypted-nonces"},{"type":"function","title":"NoNoncense.encrypted_nonce/3","doc":"Generate a new nonce and encrypt it. This creates an unpredictable but still unique nonce.\n\n    iex> key = :crypto.strong_rand_bytes 24\n    <<76, 201, 87, 221, 39, 41, 231, 66, 80, 199, 18, 164, 248, 5, 92, 42, 246, 73,\n      151, 198, 51, 190, 81, 82>>\n    iex> NoNoncense.encrypted_nonce(64, key)\n    <<50, 231, 215, 98, 233, 96, 157, 205>>\n    iex> NoNoncense.encrypted_nonce(96, key)\n    <<6, 138, 218, 96, 131, 136, 51, 242, 0, 0, 0, 0>>\n    iex> key = :crypto.strong_rand_bytes 32\n    <<175, 189, 46, 130, 235, 88, 83, 220, 44, 179, 255, 75, 255, 212, 9, 148, 53,\n      211, 157, 137, 52, 48, 247, 155, 222, 130, 70, 227, 57, 89, 137, 171>>\n    iex> NoNoncense.encrypted_nonce(128, key)\n    <<162, 10, 94, 4, 91, 56, 147, 198, 46, 87, 142, 197, 128, 41, 79, 209>>","ref":"NoNoncense.html#encrypted_nonce/3"},{"type":"function","title":"NoNoncense.get_datetime/2","doc":"Get the timestamp of the nonce as a `DateTime`, given the epoch of the instance.\nThis should only be used for `sortable_nonce/2` nonces.","ref":"NoNoncense.html#get_datetime/2"},{"type":"function","title":"NoNoncense.init/1","doc":"Initialize a nonce factory. Multiple instances with different names, epochs and even machine IDs are supported.","ref":"NoNoncense.html#init/1"},{"type":"function","title":"Options - NoNoncense.init/1","doc":"* `:machine_id` - machine ID of the node\n  * `:name` - The name of the nonce factory (default: module name).\n  * `:epoch` - Override the configured epoch for this factory instance. Defaults to the NoNoncense epoch (2025-01-01 00:00:00Z).","ref":"NoNoncense.html#init/1-options"},{"type":"function","title":"Examples - NoNoncense.init/1","doc":"iex> NoNoncense.init(machine_id: 1)\n    :ok\n\n    iex> NoNoncense.init(machine_id: 1, name: :custom, epoch: 1609459200000)\n    :ok","ref":"NoNoncense.html#init/1-examples"},{"type":"type","title":"NoNoncense.init_opts/0","doc":"","ref":"NoNoncense.html#t:init_opts/0"},{"type":"type","title":"NoNoncense.nonce/0","doc":"","ref":"NoNoncense.html#t:nonce/0"},{"type":"function","title":"NoNoncense.nonce/2","doc":"Generate a new 64/96/128-bits counter-like nonce.","ref":"NoNoncense.html#nonce/2"},{"type":"function","title":"Examples - NoNoncense.nonce/2","doc":"iex> nonce(64)\n    <<101, 6, 25, 181, 192, 128, 32, 17>>\n\n    iex> nonce(96)\n    <<101, 6, 25, 181, 192, 128, 32, 0, 0, 0, 0, 18>>\n\n    iex> nonce(128)\n    <<101, 6, 25, 181, 192, 128, 32, 0, 0, 0, 0, 0, 0, 0, 0, 19>>","ref":"NoNoncense.html#nonce/2-examples"},{"type":"type","title":"NoNoncense.nonce_size/0","doc":"","ref":"NoNoncense.html#t:nonce_size/0"},{"type":"function","title":"NoNoncense.sortable_nonce/2","doc":"Generate a nonce that is sortable by generation time, like a Snowflake ID. The first 42 bits contain the timestamp.\n\nThese nonces are *not* suitable for cryptographic purposes because they are predictable and leak their generation timestamp.","ref":"NoNoncense.html#sortable_nonce/2"},{"type":"function","title":"Examples - NoNoncense.sortable_nonce/2","doc":"iex> NoNoncense.sortable_nonce(64)\n    <<0, 15, 27, 213, 143, 128, 0, 0>>\n    iex> NoNoncense.sortable_nonce(96)\n    <<0, 15, 27, 215, 172, 0, 0, 0, 0, 0, 0, 0>>\n    iex> NoNoncense.sortable_nonce(128)\n    <<0, 15, 27, 217, 161, 128, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0>>\n\n    # the generation time can be extracted\n    iex> < > = <<0, 15, 27, 213, 143, 128, 0, 0>>\n    iex> epoch = ~U[2025-01-01T00:00:00Z] |> DateTime.to_unix(:millisecond)\n    iex> DateTime.from_unix!(ts + epoch, :millisecond)\n    ~U[2025-01-12 17:38:49.534Z]","ref":"NoNoncense.html#sortable_nonce/2-examples"},{"type":"module","title":"NoNoncense.MachineId","doc":"Determine unique machine IDs for nodes, which can be used to guarantee uniqueness in a distributed system.\n\nTo determine the unique node ID, you must provide a list of possible node identifiers. The possible identifiers are IP addresses, OTP node identifiers, hostnames and fully-qualified domain names. You should only provide possible identifiers that can't conflict with one another. So if you don't explicitly set OTP node names you should not add the default `:\"nonode@nohost\"` to your identifier list.\n\n> #### Use the same node list everywhere {: .warning}\n>\n> Your `node_list` must be the same for every node or the generated machine IDs will not be unique.\n\nYou can configure the options in your application environment and just pass them into `id/1`.\n\nAfter https://github.com/blitzstudios/snowflake","ref":"NoNoncense.MachineId.html"},{"type":"function","title":"NoNoncense.MachineId.host_identifiers/0","doc":"Get a list of all identifiers of the current node. You can use one or more of these values to populate your node list.","ref":"NoNoncense.MachineId.html#host_identifiers/0"},{"type":"function","title":"Examples / doctests - NoNoncense.MachineId.host_identifiers/0","doc":"iex> MachineId.host_identifiers()\n    [:nonode@nohost, \"host.mydomain.com\", \"10.11.12.13\", \"myhost\", \"fe80::1234::abcd\"]","ref":"NoNoncense.MachineId.html#host_identifiers/0-examples-doctests"},{"type":"type","title":"NoNoncense.MachineId.host_identifiers/0","doc":"","ref":"NoNoncense.MachineId.html#t:host_identifiers/0"},{"type":"function","title":"NoNoncense.MachineId.id!/1","doc":"Determine the current node's machine ID.","ref":"NoNoncense.MachineId.html#id!/1"},{"type":"function","title":"Examples / doctests - NoNoncense.MachineId.id!/1","doc":"# provide a list of possible node identifiers\n    iex> node_list = [\"1.1.1.1\", \"127.0.0.1\", \"8.8.8.8\", \"0.0.0.0\"]\n    iex> MachineId.id!(node_list: node_list)\n    2\n\n    # a statically configured ID will override the node list\n    iex> node_list = [\"1.1.1.1\", \"127.0.0.1\", \"8.8.8.8\", \"0.0.0.0\"]\n    iex> MachineId.id!(machine_id: 1, node_list: node_list)\n    1\n\n    # the node ID must be within the provided range (default 0-1023)\n    iex> node_list = [\"1.1.1.1\", \"127.0.0.1\", \"8.8.8.8\", \"0.0.0.0\"]\n    iex> MachineId.id!(max_nodes: 2, node_list: node_list)\n    ** (RuntimeError) Node ID 2 out of range 0-1\n\n    # raises when the machine ID could not be determined from the node list\n    iex> node_list = [\"1.1.1.1\"]\n    iex> MachineId.id!(node_list: node_list)\n    ** (RuntimeError) machine ID could not be determined","ref":"NoNoncense.MachineId.html#id!/1-examples-doctests"},{"type":"type","title":"NoNoncense.MachineId.id_opts/0","doc":"","ref":"NoNoncense.MachineId.html#t:id_opts/0"},{"type":"module","title":"NoNoncense.MachineId.ConflictGuard","doc":"Guards against machine ID conflicts between nodes. If a new node joins the cluster with the same ID, it is sent the machine IDs of all existing nodes, will become aware of the ID conflict and will call the `on_conflict` callback that can take action to prevent bad stuff from happening (for example, that the uniqueness guarantee of `NoNoncense` will no longer hold).\n\nBy default, the `on_conflict` callback emergeny shuts down the entire node using `:erlang.halt/1` with status code 111.\n\nOf course, all of this only works if the nodes are actually connected to one another.","ref":"NoNoncense.MachineId.ConflictGuard.html"},{"type":"function","title":"NoNoncense.MachineId.ConflictGuard.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"NoNoncense.MachineId.ConflictGuard.html#child_spec/1"},{"type":"type","title":"NoNoncense.MachineId.ConflictGuard.opts/0","doc":"","ref":"NoNoncense.MachineId.ConflictGuard.html#t:opts/0"},{"type":"function","title":"NoNoncense.MachineId.ConflictGuard.start_link/1","doc":"Let's get this puppy going!","ref":"NoNoncense.MachineId.ConflictGuard.html#start_link/1"},{"type":"extras","title":"NoNoncense","doc":"# NoNoncense\n\nGenerate locally unique nonces (number-only-used-once) in distributed Elixir.\n\nNonces come in multiple varians:\n\n- counter nonces that are unique but predictable and can be generated incredibly quickly\n- sortable nonces ([Snowflake IDs](https://en.wikipedia.org/wiki/Snowflake_ID)) that have an accurate creation timestamp in their first bits\n- encrypted nonces that are unique but unpredictable","ref":"readme.html"},{"type":"extras","title":"Installation - NoNoncense","doc":"The package is hosted on [hex.pm](https://hex.pm/packages/no_noncense) and can be installed by adding `:no_noncense` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:no_noncense, \"~> 0.0.3\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Docs - NoNoncense","doc":"Documentation can be found on [hexdocs.pm](https://hexdocs.pm/no_noncense/).","ref":"readme.html#docs"},{"type":"extras","title":"Usage - NoNoncense","doc":"Note that `NoNoncense` is not a GenServer. Instead, it stores its initial state using `m::persistent_term` and its internal counter using `m::atomics`. Because `m::persistent_term` triggers a garbage collection cycle on writes, it is recommended to initialize your `NoNoncense` instance(s) at application start, when there is hardly any garbage to collect.\n\n```elixir\n# lib/my_app/application.ex\n# generate a machine ID, start conflict guard and initialize a NoNoncense instance\ndefmodule MyApp.Application do\n  use Application\n\n  def start(_type, _args) do\n    # grab your node_list from your application environment\n    machine_id = NoNoncense.MachineId.id!(node_list: [:\"myapp@127.0.0.1\"])\n    :ok = NoNoncense.init(machine_id: machine_id)\n\n    children =\n      [\n        # optional but recommended\n        {NoNoncense.MachineId.ConflictGuard, [machine_id: machine_id]}\n      ]\n\n    opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\nend\n```\n\nThen you can generate nonces.\n\n```elixir\n# generate counter nonces\niex> <<_::64>> = NoNoncense.nonce(64)\niex> <<_::96>> = NoNoncense.nonce(96)\niex> <<_::128>> = NoNoncense.nonce(128)\n\n# generate sortable nonces\niex> <<_::64>> = NoNoncense.sortable_nonce(64)\niex> <<_::96>> = NoNoncense.sortable_nonce(96)\niex> <<_::128>> = NoNoncense.sortable_nonce(128)\n\n# generate encrypted nonces\n# be sure to read the NoNoncense docs before using 64/96 bits encrypted nonces\niex> <<_::64>> = NoNoncense.encrypted_nonce(64, :crypto.strong_rand_bytes(24))\niex> <<_::96>> = NoNoncense.encrypted_nonce(96, :crypto.strong_rand_bytes(24))\niex> <<_::128>> = NoNoncense.encrypted_nonce(128, :crypto.strong_rand_bytes(32))\n```","ref":"readme.html#usage"},{"type":"extras","title":"Benchmarks - NoNoncense","doc":"```\nOn Debian Bookworm, AMD 9700X (8C 16T), 32GB, 990 Pro.\n\nnonce(128) single:             54_981_091 ops/s\nnonce(96) single:              44_348_625 ops/s\nnonce(128) multi:              37_745_241 ops/s\nnonce(64) multi:               37_617_656 ops/s\nnonce(96) multi:               37_552_652 ops/s\nsortable_nonce(96) multi:      35_124_078 ops/s\nsortable_nonce(128) multi:     34_819_544 ops/s\nnonce(64) single:              25_847_171 ops/s\nsortable_nonce(128) single:    21_776_095 ops/s\nsortable_nonce(96) single:     21_016_157 ops/s\nencrypted_nonce(128) multi:    13_485_052 ops/s\nencrypted_nonce(64) multi:     11_049_795 ops/s\nencrypted_nonce(96) multi:     10_438_713 ops/s\nsortable_nonce(64) multi:       8_192_779 ops/s\nsortable_nonce(64) single:      8_191_835 ops/s\nstrong_rand_bytes(8) multi:     4_859_998 ops/s\nstrong_rand_bytes(12) multi:    4_856_265 ops/s\nstrong_rand_bytes(16) multi:    4_807_786 ops/s\nstrong_rand_bytes(8) single:    2_731_625 ops/s\nstrong_rand_bytes(12) single:   2_723_526 ops/s\nstrong_rand_bytes(16) single:   2_723_058 ops/s\nencrypted_nonce(128) single:    2_446_565 ops/s\nencrypted_nonce(64) single:     1_209_200 ops/s\nencrypted_nonce(96) single:     1_118_734 ops/s\n```\n\nSome things of note:\n\n- NoNoncense nonces generate much faster than random binaries.\n- All methods are quick enough to handle very high peak loads.\n- The plain (counter) nonce generation rate is hardly influenced by multithreading and seems to hit a bottleneck of some kind, probably to do with `:persistent_term` or `:atomics`. Still, it hits a really high rate.\n- Encrypting the nonce exacts a very hefty penalty, but parallellization scales well to alleviate the issue.\n- Triple DES sucks.","ref":"readme.html#benchmarks"},{"type":"extras","title":"LICENSE","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS","ref":"license.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.1"}}